#ifndef ADM8211_H
#define ADM8211_H

#include "ieee80211.h"

/* ADM8211 Registers */

/* CR (PCI Configuration Registers); these can be accessed with
 * pci_{read,write}_config_{byte,dword} functions */
#define ADM8211_CR_LID		0x00 /* CR0 */
#define ADM8211_CR_CSC		0x04 /* CR1 */
#define ADM8211_CR_CC		0x08 /* CR2 */
#define ADM8211_CR_LT		0x0C /* CR3 */
#define ADM8211_CR_IOBA		0x10 /* CR4 */
#define ADM8211_CR_MBA		0x14 /* CR5 */
#define ADM8211_CR_CIS		0x28 /* CR10 */
#define ADM8211_CR_SID		0x2C /* CR11 */
#define ADM8211_CR_CP		0x34 /* CR13 */
#define ADM8211_CR_CI		0x3C /* CR15 */
#define ADM8211_CR_DS		0x40 /* CR16 */
#define ADM8211_CR_SIG		0x80 /* CR32 */
#define ADM8211_CR_PMR0		0xC0 /* CR48 */
#define ADM8211_CR_PMR1		0xC4 /* CR49 */

/* CR1 - CSC */
#define ADM8211_CSC_SPE		(1 << 31)
#define ADM8211_CSC_SES		(1 << 30)
#define ADM8211_CSC_SMA		(1 << 29)
#define ADM8211_CSC_STA		(1 << 28)
#define ADM8211_CSC_SDST	((1 << 26) | (1 << 25))
#define ADM8211_CSC_SDPR	(1 << 24)
#define ADM8211_CSC_SFBB	(1 << 23)
#define ADM8211_CSC_NC		(1 << 20)
#define ADM8211_CSC_CSE		(1 << 8)
#define ADM8211_CSC_CPE		(1 << 6)
#define ADM8211_CSC_CMO		(1 << 2)
#define ADM8211_CSC_CMSA	(1 << 1)
#define ADM8211_CSC_CIOSA	(1 << 0)

/* CR32 (SIG) signature */
#define ADM8211_SIG1		0x82011317
#define ADM8211_SIG2		0x82111317


/* CSR (Host Control and Status Registers) */
#define ADM8211_CSR_PAR		0x00 /* CSR0 */
#define ADM8211_CSR_FRCTL	0x04 /* CSR0A */
#define ADM8211_CSR_TDR		0x08 /* CSR1 */
#define ADM8211_CSR_WTDP	0x0C /* CSR1A */
#define ADM8211_CSR_RDR		0x10 /* CSR2 */
#define ADM8211_CSR_WRDP	0x14 /* CSR2A */
#define ADM8211_CSR_RDB		0x18 /* CSR3 */
#define ADM8211_CSR_TDBD	0x20 /* CSR4 */
#define ADM8211_CSR_TDBP	0x24 /* CSR4A */
#define ADM8211_CSR_STSR	0x28 /* CSR5 */
#define ADM8211_CSR_NAR		0x30 /* CSR6 */
#define ADM8211_CSR_IER		0x38 /* CSR7 */
#define ADM8211_CSR_LPC		0x40 /* CSR8 */
#define ADM8211_CSR_CSR_TEST1	0x44 /* CSR8A */
#define ADM8211_CSR_SPR		0x48 /* CSR9 */
#define ADM8211_CSR_CSR_TEST0	0x4C /* CSR9A */
#define ADM8211_CSR_WCSR	0x50 /* CSR10 */
#define ADM8211_CSR_WPDR	0x54 /* CSR10A */
#define ADM8211_CSR_GPTMR	0x58 /* CSR11 */
#define ADM8211_CSR_GPIO	0x5C /* CSR11A */
#define ADM8211_CSR_BBPCTL	0x60 /* CSR12 */
#define ADM8211_CSR_SYNCTL	0x64 /* CSR12A */
#define ADM8211_CSR_PLCPHD	0x68 /* CSR13 */
#define ADM8211_CSR_MMIWA	0x6C /* CSR13A */
#define ADM8211_CSR_MMIRD0	0x70 /* CSR14 */
#define ADM8211_CSR_MMIRD1	0x74 /* CSR14A */
#define ADM8211_CSR_TXBR	0x78 /* CSR15 */
#define ADM8211_CSR_TOFS2	0x84 /* CSR17 */
#define ADM8211_CSR_CMDR	0x88 /* CSR18 */
#define ADM8211_CSR_PCIC	0x8C /* CSR19 */
#define ADM8211_CSR_PMCSR	0x90 /* CSR20 */
#define ADM8211_CSR_PAR0	0x94 /* CSR21 */
#define ADM8211_CSR_PAR1	0x98 /* CSR22 */
#define ADM8211_CSR_MAR0	0x9C /* CSR23 */
#define ADM8211_CSR_MAR1	0xA0 /* CSR24 */
#define ADM8211_CSR_ATIMDA0	0xA4 /* CSR25 */
#define ADM8211_CSR_ABDA1	0xA8 /* CSR26 */
#define ADM8211_CSR_BSSID0	0xAC /* CSR27 */
#define ADM8211_CSR_TXLMT	0xB0 /* CSR28 */
#define ADM8211_CSR_MIBCNT	0xB4 /* CSR29 */
#define ADM8211_CSR_BCNT	0xB8 /* CSR30 */
#define ADM8211_CSR_TSFTH	0xBC /* CSR31 */
#define ADM8211_CSR_TSC		0xC0 /* CSR32 */
#define ADM8211_CSR_SYNRF	0xC4 /* CSR33 */
#define ADM8211_CSR_BPLI	0xC8 /* CSR34 */
#define ADM8211_CSR_CAP0	0xCC /* CSR35 */
#define ADM8211_CSR_CAP1	0xD0 /* CSR36 */
#define ADM8211_CSR_RMD		0xD4 /* CSR37 */
#define ADM8211_CSR_CFPP	0xD8 /* CSR38 */
#define ADM8211_CSR_TOFS0	0xDC /* CSR39 */
#define ADM8211_CSR_TOFS1	0xE0 /* CSR40 */
#define ADM8211_CSR_IFST	0xE4 /* CSR41 */
#define ADM8211_CSR_RSPT	0xE8 /* CSR42 */
#define ADM8211_CSR_TSFTL	0xEC /* CSR43 */
#define ADM8211_CSR_WEPCTL	0xF0 /* CSR44 */
#define ADM8211_CSR_WESK	0xF4 /* CSR45 */
#define ADM8211_CSR_WEPCNT	0xF8 /* CSR46 */
#define ADM8211_CSR_MACTEST	0xFC /* CSR47 */
#define ADM8211_CSR_FER		0x100
#define ADM8211_CSR_FEMR	0x104
#define ADM8211_CSR_FPSR	0x108
#define ADM8211_CSR_FFER	0x10C

/* CSR0 - PAR (PCI Address Register) */
#define ADM8211_PAR_MWIE	(1 << 24)
#define ADM8211_PAR_MRLE	(1 << 23)
#define ADM8211_PAR_MRME	(1 << 21)
#define ADM8211_PAR_RAP		((1 << 18) | (1 << 17))
#define ADM8211_PAR_CAL		((1 << 15) | (1 << 14))
#define ADM8211_PAR_PBL		0x00003f00
#define ADM8211_PAR_BLE		(1 << 7)
#define ADM8211_PAR_DSL		0x0000007c
#define ADM8211_PAR_BAR		(1 << 1)
#define ADM8211_PAR_SWR		(1 << 0)

/* CSR1 - FRCTL (Frame Control Register) */
#define ADM8211_FRCTL_PWRMGT	(1 << 31)
#define ADM8211_FRCTL_MAXPSP	(1 << 27)
#define ADM8211_FRCTL_AID	0x0000ffff
#define ADM8211_FRCTL_AID_ON	0x0000c000

/* CSR5 - STSR (Status Register) */
#define ADM8211_STSR_PCF	(1 << 31)
#define ADM8211_STSR_BCNTC	(1 << 30)
#define ADM8211_STSR_GPINT	(1 << 29)
#define ADM8211_STSR_LinkOff	(1 << 28)
#define ADM8211_STSR_ATIMTC	(1 << 27)
#define ADM8211_STSR_TSFTF	(1 << 26)
#define ADM8211_STSR_TSCZ	(1 << 25)
#define ADM8211_STSR_LinkOn	(1 << 24)
#define ADM8211_STSR_SQL	(1 << 23)
#define ADM8211_STSR_WEPTD	(1 << 22)
#define ADM8211_STSR_ATIME	(1 << 21)
#define ADM8211_STSR_TBTT	(1 << 20)
#define ADM8211_STSR_NISS	(1 << 16)
#define ADM8211_STSR_AISS	(1 << 15)
#define ADM8211_STSR_TEIS	(1 << 14)
#define ADM8211_STSR_FBE	(1 << 13)
#define ADM8211_STSR_REIS	(1 << 12)
#define ADM8211_STSR_GPTT	(1 << 11)
#define ADM8211_STSR_RPS	(1 << 8)
#define ADM8211_STSR_RDU	(1 << 7)
#define ADM8211_STSR_RCI	(1 << 6)
#define ADM8211_STSR_TUF	(1 << 5)
#define ADM8211_STSR_TRT	(1 << 4)
#define ADM8211_STSR_TLT	(1 << 3)
#define ADM8211_STSR_TDU	(1 << 2)
#define ADM8211_STSR_TPS	(1 << 1)
#define ADM8211_STSR_TCI	(1 << 0)

/* CSR6 - NAR (Network Access Register) */
#define ADM8211_NAR_TXCF	(1 << 31)
#define ADM8211_NAR_HF		(1 << 30)
#define ADM8211_NAR_UTR		(1 << 29)
#define ADM8211_NAR_SQ		(1 << 28)
#define ADM8211_NAR_CFP		(1 << 27)
#define ADM8211_NAR_SF		(1 << 21)
#define ADM8211_NAR_TR		((1 << 15) | (1 << 14))
#define ADM8211_NAR_ST		(1 << 13)
#define ADM8211_NAR_OM		((1 << 11) | (1 << 10))
#define ADM8211_NAR_MM		(1 << 7)
#define ADM8211_NAR_PR		(1 << 6)
#define ADM8211_NAR_EA		(1 << 5)
#define ADM8211_NAR_PB		(1 << 3)
#define ADM8211_NAR_STPDMA	(1 << 2)
#define ADM8211_NAR_SR		(1 << 1)
#define ADM8211_NAR_CTX		(1 << 0)

/* CSR7 - IER (Interrupt Enable Register) */
#define ADM8211_IER_PCFIE	(1 << 31)
#define ADM8211_IER_BCNTCIE	(1 << 30)
#define ADM8211_IER_GPIE	(1 << 29)
#define ADM8211_IER_LinkOffIE	(1 << 28)
#define ADM8211_IER_ATIMTCIE	(1 << 27)
#define ADM8211_IER_TSFTFIE	(1 << 26)
#define ADM8211_IER_TSCZE	(1 << 25)
#define ADM8211_IER_LinkOnIE	(1 << 24)
#define ADM8211_IER_SQLIE	(1 << 23)
#define ADM8211_IER_WEPIE	(1 << 22)
#define ADM8211_IER_ATIMEIE	(1 << 21)
#define ADM8211_IER_TBTTIE	(1 << 20)
#define ADM8211_IER_NIE		(1 << 16)
#define ADM8211_IER_AIE		(1 << 15)
#define ADM8211_IER_TEIE	(1 << 14)
#define ADM8211_IER_FBEIE	(1 << 13)
#define ADM8211_IER_REIE	(1 << 12)
#define ADM8211_IER_GPTIE	(1 << 11)
#define ADM8211_IER_RSIE	(1 << 8)
#define ADM8211_IER_RUIE	(1 << 7)
#define ADM8211_IER_RCIE	(1 << 6)
#define ADM8211_IER_TUIE	(1 << 5)
#define ADM8211_IER_TRTIE	(1 << 4)
#define ADM8211_IER_TLTTIE	(1 << 3)
#define ADM8211_IER_TDUIE	(1 << 2)
#define ADM8211_IER_TPSIE	(1 << 1)
#define ADM8211_IER_TCIE	(1 << 0)


/* CSR9 - SPR (Serial Port Register) */
#define ADM8211_SPR_SRS		(1 << 11)
#define ADM8211_SPR_SDO		(1 << 3)
#define ADM8211_SPR_SDI		(1 << 2)
#define ADM8211_SPR_SCLK	(1 << 1)
#define ADM8211_SPR_SCS		(1 << 0)

/* CSR9A - CSR_TEST0 */
#define ADM8211_CSR_TEST0_EPNE	(1 << 18)
#define ADM8211_CSR_TEST0_EPSNM	(1 << 17)
#define ADM8211_CSR_TEST0_EPTYP	(1 << 16)
#define ADM8211_CSR_TEST0_EPRLD	(1 << 15)

/* CSR11A - GPIO */
#define ADM8211_CSR_GPIO_EN5	(1 << 17)
#define ADM8211_CSR_GPIO_EN4	(1 << 16)
#define ADM8211_CSR_GPIO_EN3	(1 << 15)
#define ADM8211_CSR_GPIO_EN2	(1 << 14)
#define ADM8211_CSR_GPIO_EN1	(1 << 13)
#define ADM8211_CSR_GPIO_EN0	(1 << 12)
#define ADM8211_CSR_GPIO_O5	(1 << 11)
#define ADM8211_CSR_GPIO_O4	(1 << 10)
#define ADM8211_CSR_GPIO_O3	(1 << 9)
#define ADM8211_CSR_GPIO_O2	(1 << 8)
#define ADM8211_CSR_GPIO_O1	(1 << 7)
#define ADM8211_CSR_GPIO_O0	(1 << 6)
#define ADM8211_CSR_GPIO_IN	0x0000003f

/* CSR12 - BBPCTL (BBP Control port) */
#define ADM8211_BBPCTL_WR	(1 << 17)
#define ADM8211_BBPCTL_RD	(1 << 16)
#define ADM8211_BBPCTL_ADDR	0x0000ff00
#define ADM8211_BBPCTL_DATA	0x000000ff

/* CSR12A - SYNCTL (Synthesizer Control port) */
#define ADM8211_SYNCTL_WR	(1 << 31)
#define ADM8211_SYNCTL_RD	(1 << 30)
#define ADM8211_SYNCTL_CS0	(1 << 29)
#define ADM8211_SYNCTL_CS1	(1 << 28)
#define ADM8211_SYNCTL_CAL	(1 << 27)
#define ADM8211_SYNCTL_SELCAL	(1 << 26)
#define ADM8211_SYNCTL_RFtype	((1 << 24) || (1 << 23) || (1 << 22))
#define ADM8211_SYNCTL_RFDM	(1 << 22)
/* SYNCTL 21:0 Data (Si4126: 18-bit data, 4-bit address) */

/* CSR18 - CMDR (Command Register) */
#define ADM8211_CMDR_PM		(1 << 19)
#define ADM8211_CMDR_APM	(1 << 18)
#define ADM8211_CMDR_RTE	(1 << 4)
#define ADM8211_CMDR_DRT	((1 << 3) | (1 << 2))
#define ADM8211_CMDR_DRT_SF	(1 << 3)

/* CSR33 - SYNRF (SYNRF direct control) */
#define ADM8211_SYNRF_SELSYN	(1 << 31)
#define ADM8211_SYNRF_SELRF	(1 << 30)
#define ADM8211_SYNRF_LERF	(1 << 29)
#define ADM8211_SYNRF_LEIF	(1 << 28)
#define ADM8211_SYNRF_SYNCLK	(1 << 27)
#define ADM8211_SYNRF_SYNDATA	(1 << 26)
#define ADM8211_SYNRF_PE1	(1 << 25)
#define ADM8211_SYNRF_PE2	(1 << 24)
#define ADM8211_SYNRF_PA_PE	(1 << 23)
#define ADM8211_SYNRF_PHYRST	(1 << 18)

/* CSR44 - WEPCTL (WEP Control) */
#define ADM8211_WEPCTL_WEPENABLE   (1 << 31)
#define ADM8211_WEPCTL_AUTO_SWITCH (1 << 30)
#define ADM8211_WEPCTL_CURRENT_TABLE (1 << 29)
#define ADM8211_WEPCTL_TABLE_WR	(1 << 28)
#define ADM8211_WEPCTL_TABLE_RD	(1 << 27)
#define ADM8211_WEPCTL_WEPRXBYP	(1 << 25)
#define ADM8211_WEPCTL_ADDR	(0x000001ff)

/* CSR45 - WESK (Data Entry for Share/Individual Key) */
#define ADM8211_WESK_DATA	(0x0000ffff)

/* FER (Function Event Register) */
#define ADM8211_FER_INTR_EV_ENT	(1 << 15)



#define ADM8211_CSR_READ(r) \
readl(priv->mem + ADM8211_CSR_ ## r)
#define ADM8211_CSR_WRITE(r, val) \
writel((val), priv->mem + ADM8211_CSR_ ## r)


/* Si4126 RF Synthesizer - Control Registers */
#define SI4126_MAIN_CONF	0
#define SI4126_PHASE_DET_GAIN	1
#define SI4126_POWERDOWN	2
#define SI4126_RF1_N_DIV	3 /* only Si4136 */
#define SI4126_RF2_N_DIV	4
#define SI4126_IF_N_DIV		5
#define SI4126_RF1_R_DIV	6 /* only Si4136 */
#define SI4126_RF2_R_DIV	7
#define SI4126_IF_R_DIV		8

/* Main Configuration */
#define SI4126_MAIN_XINDIV2	(1 << 6)
#define SI4126_MAIN_IFDIV	((1 << 11) | (1 << 10))
/* Powerdown */
#define SI4126_POWERDOWN_PDIB	(1 << 1)
#define SI4126_POWERDOWN_PDRB	(1 << 0)


/* RF3000 BBP - Control Port Registers */
/* 0x00 - reserved */
#define RF3000_MODEM_CTRL__RX_STATUS 0x01
#define RF3000_CCA_CTRL 0x02
#define RF3000_DIVERSITY__RSSI 0x03
#define RF3000_RX_SIGNAL_FIELD 0x04
#define RF3000_RX_LEN_MSB 0x05
#define RF3000_RX_LEN_LSB 0x06
#define RF3000_RX_SERVICE_FIELD 0x07
#define RF3000_TX_VAR_GAIN__TX_LEN_EXT 0x11
#define RF3000_TX_LEN_MSB 0x12
#define RF3000_TX_LEN_LSB 0x13
#define RF3000_LOW_GAIN_CALIB 0x14
#define RF3000_HIGH_GAIN_CALIB 0x15

/* ADM8211 revisions */
#define ADM8211_REV_AB 0x11
#define ADM8211_REV_AF 0x15
#define ADM8211_REV_BA 0x20
#define ADM8211_REV_CA 0x30

#define TX_RING_SIZE 16
#define RX_RING_SIZE 16

struct adm8211_ring_info {
	struct sk_buff *skb;
	dma_addr_t mapping;
};


struct adm8211_priv {
	struct pci_dev *pdev;
	spinlock_t lock;
	int io_base; /* PCI Region 0: I/O ports */
	void *mem; /* PCI Region 1: Memory */
	struct adm8211_desc *rx_ring;
	struct adm8211_desc *tx_ring;
	dma_addr_t rx_ring_dma;
	dma_addr_t tx_ring_dma;
	struct adm8211_ring_info rx_buffers[RX_RING_SIZE];
	struct adm8211_ring_info tx_buffers[TX_RING_SIZE];
	unsigned cur_tx, dirty_tx, cur_rx;
	struct sk_buff_head rx_queue;
	struct tasklet_struct rx_tasklet;
#ifdef ADM8211_PROC
	struct proc_dir_entry *proc;
#endif
	int iw_mode; /* operating mode (IW_MODE_*) */

	struct net_device_stats stats;
	struct iw_statistics wstats;
	struct ieee80211_data ieee80211;
	unsigned long scan_timestamp;

	int (*eth_header_parse)(struct sk_buff *skb, unsigned char *haddr);

	u8 plcp_signal;
	u16 frag_thresh;

	u8 *eeprom;
	size_t eeprom_len;

	u8 revid;

#define ADM8211_TYPE_INTERSIL	0x00
#define ADM8211_TYPE_RFMD	0x01
#define ADM8211_TYPE_MARVEL	0x02
	unsigned int rf_type:3;
	unsigned int bbp_type:3;
};

static const struct ieee80211_chan_range cranges[] = {
	{1,  11},	/* FCC */
	{1,  11},	/* IC */
	{1,  13},	/* ETSI */
	{10, 11},	/* SPAIN */
	{10, 13},	/* FRANCE */
	{14, 14},	/* MMK */
	{1,  14},	/* MMK2 */
};

int adm8211_rf_set_channel(struct net_device *dev, int channel);
int adm8211_80211_header_parse(struct sk_buff *skb, unsigned char *haddr);
void adm8211_write_wepkey(struct net_device *dev, int index);
void adm8211_update_wep(struct net_device *dev);

#endif /* ADM8211_H */
